import 'package:flutter/material.dart';

/// [GuideInterval] model used to specify the grid guides for a Widget
class GuideInterval {
  /// The distance between the primary lines in the grid, in logical pixels.
  final int intervals;

  /// The number of major divisions within each primary grid cell.
  /// This is the number of major divisions per [interval], including the
  /// primary grid's line.
  ///
  /// If this is set to 2, then for each [interval] there will be two divisions
  /// separated by a line
  final int divisions;

  /// [LineDecorator] used to define interval line width and color
  final LineDecorator intervalDecorator;

  /// [LineDecorator] used to define division line width and color
  final LineDecorator divisionDecorator;
  const GuideInterval(
    this.intervals, {
    this.divisions = 1,
    this.intervalDecorator =
        const LineDecorator(color: Color(0x7FC3E8F3), width: 0.5),
    this.divisionDecorator =
        const LineDecorator(color: Color(0x7FC3E8F3), width: 0.25),
  }) : assert(divisions > 0,
            'The "divisions" property must be greater than zero. If there were no divisions, the grid paper would not paint anything.');

  /// creates a new [GuideInterval] with provided overrides
  GuideInterval copyWith({
    int? intervals,
    int? divisions,
    LineDecorator? intervalDecorator,
    LineDecorator? divisionDecorator,
  }) {
    return GuideInterval(
      intervals ?? this.intervals,
      divisions: divisions ?? this.divisions,
      intervalDecorator: intervalDecorator ?? this.intervalDecorator,
      divisionDecorator: divisionDecorator ?? this.divisionDecorator,
    );
  }

  @override
  String toString() =>
      'GridInterval(intervals: $intervals, divisions: $divisions, intervalDecorator: $intervalDecorator, divisionDecorator: $divisionDecorator)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is GuideInterval &&
        other.intervals == intervals &&
        other.divisions == divisions &&
        other.intervalDecorator == intervalDecorator &&
        other.divisionDecorator == divisionDecorator;
  }

  @override
  int get hashCode =>
      intervals.hashCode ^
      divisions.hashCode ^
      intervalDecorator.hashCode ^
      divisionDecorator.hashCode;
}

/// [Keyline], used to specify the [Design keylines](https://material.io/archive/guidelines/layout/metrics-keylines.html#metrics-keylines-keylines-spacing) for a Widget
class Keyline {
  /// Position of keyline relative edge
  final double xPos;

  /// Edge from which [xPos] is defined
  final KeylineGravity gravity;

  /// [KeylineDecorator] used to define keyline and keyline margins colors and width
  final KeylineDecorator decorator;

  const Keyline(
    this.xPos, {
    this.gravity = KeylineGravity.start,
    this.decorator = const KeylineDecorator(),
  });

  /// creates a new [Keyline] with provided overrides
  Keyline copyWith({
    double? xPos,
    KeylineGravity? gravity,
    KeylineDecorator? decorator,
  }) {
    return Keyline(
      xPos ?? this.xPos,
      gravity: gravity ?? this.gravity,
      decorator: decorator ?? this.decorator,
    );
  }

  @override
  String toString() =>
      'Keyline(xPos: $xPos, gravity: $gravity, decorator: $decorator)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is Keyline &&
        other.xPos == xPos &&
        other.gravity == gravity &&
        other.decorator == decorator;
  }

  @override
  int get hashCode => xPos.hashCode ^ gravity.hashCode ^ decorator.hashCode;
}

/// [KeylineGravity], edge from which [Keyline] is defined
enum KeylineGravity { start, end }

/// [KeylineDecorator] used to define keyline styles/decoration
class KeylineDecorator {
  /// [LineDecorator] used to define keylines width and color
  final LineDecorator? lineDecorator;

  /// [BlockLineDecorator] used to define keyline margins width and gradient
  final BlockLineDecorator? marginDecorator;
  const KeylineDecorator({
    this.lineDecorator =
        const LineDecorator(color: Color(0x77EC018C), width: 1),
    this.marginDecorator = const BlockLineDecorator(
        startColor: Color(0x5500BCD4),
        endColor: Color(0x1100BCD4),
        width: 16.0),
  });

  /// creates a new [KeylineDecorator] with provided overrides
  KeylineDecorator copyWith({
    LineDecorator? lineDecorator,
    BlockLineDecorator? marginDecorator,
  }) {
    return KeylineDecorator(
      lineDecorator: lineDecorator ?? this.lineDecorator,
      marginDecorator: marginDecorator ?? this.marginDecorator,
    );
  }

  @override
  String toString() =>
      'KeylineDecorator(lineDecorator: $lineDecorator, marginDecorator: $marginDecorator)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is KeylineDecorator &&
        other.lineDecorator == lineDecorator &&
        other.marginDecorator == marginDecorator;
  }

  @override
  int get hashCode => lineDecorator.hashCode ^ marginDecorator.hashCode;
}

/// [LineDecorator] used to define the line decoration
class LineDecorator {
  /// Line color
  final Color color;

  /// Line width
  final double width;

  const LineDecorator({
    required this.color,
    required this.width,
  });

  /// creates a new [LineDecorator] with provided overrides
  LineDecorator copyWith({
    Color? color,
    double? width,
  }) {
    return LineDecorator(
      color: color ?? this.color,
      width: width ?? this.width,
    );
  }

  @override
  String toString() => 'LineDecorator(color: $color, width: $width)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is LineDecorator &&
        other.color == color &&
        other.width == width;
  }

  @override
  int get hashCode => color.hashCode ^ width.hashCode;
}

/// [LineDecorator] used to define the block line decoration
class BlockLineDecorator {
  /// Line gradient start color (gradient across widget)
  final Color startColor;

  /// Line gradient end color (gradient across widget)
  final Color endColor;

  /// Line width
  final double width;
  const BlockLineDecorator({
    required this.startColor,
    required this.endColor,
    required this.width,
  });

  /// creates a new [BlockLineDecorator] with provided overrides
  BlockLineDecorator copyWith({
    Color? startColor,
    Color? endColor,
    double? width,
  }) {
    return BlockLineDecorator(
      startColor: startColor ?? this.startColor,
      endColor: endColor ?? this.endColor,
      width: width ?? this.width,
    );
  }

  @override
  String toString() =>
      'RectDecorator(startColor: $startColor, endColor: $endColor, width: $width)';

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;

    return other is BlockLineDecorator &&
        other.startColor == startColor &&
        other.endColor == endColor &&
        other.width == width;
  }

  @override
  int get hashCode => startColor.hashCode ^ endColor.hashCode ^ width.hashCode;
}
